/*
 * Christoph Peters
 */



/**
 * Ein Objekt dieser Klasse reprsentiert einen zusammengesetzen
 * Ausdruck, bestehend aus zwei Teilausdrcken und einem
 * Operatorsymbol.
 */
public class OperatorAusdruck extends Ausdruck {
    
    /** Teilausdruck links vom Operatorsymbol (Infix). */
    private Ausdruck linkerTeilausdruck;
    
    /** Teilausdruck rechts vom Operatorsymbol (Infix). */
    private Ausdruck rechterTeilausdruck;
    
    /** Operator diesen Ausdrucks. */
    private char operatorsymbol;
    
    /** Operatorsymbol fr die Addition. */
    private static final char PLUS = '+';
    
    /** Operatorsymbol fr die Subtraktion. */
    private static final char MINUS = '-';
    
   /** Operatorsymbol fr die Multiplikation. */
    private static final char MAL = '*';
    
    /** Operatorsymbol fr die Division. */
    private static final char GETEILT = '/';
    
    /** Leerzeichen. */;
    private static final String LEERZEICHEN = " ";
    
    /** Fehlermeldung bei einer versuchten Division durch Null. */
    private static final String DIVISIONDURCHNULL = "/ by zero";
    
    /**
     * Erzeugt einen Ausdruck fr ein Operatorsymbol und zwei
     * Teilausdrcke.
     *
     * @param teilausdruckLinks   Linker Teilausdruck fr diesen 2 Ausdruck.
     * @param operator            Operatorsymbol fr diesen Ausdruck.
     * @param teilausdruckRechts  Rechter Teilausdruck fr diesen 2 Ausdruck.
     */
    public OperatorAusdruck(Ausdruck teilausdruckLinks, char operator,
                            Ausdruck teilausdruckRechts) {
    
        this.linkerTeilausdruck = teilausdruckLinks;
        this.rechterTeilausdruck = teilausdruckRechts;
        this.operatorsymbol = operator;
    }
    
    
   /**
   * Wertet diesen Ausdruck auf der Basis der angegebenen
   * Variablenbelegung aus.
   *
   * @param belegung  Variablenbelegung, auf deren Basis
   *        dieser Ausdruck ausgewertet wird
   * @return Wert dieses Ausdrucks
   * @throws UnbekannteVariableException  wird geworfen, wenn
   *         eine Variable des Ausdrucks in der Belegung keinen
   *         zugeordneten Wert besitzt
   * @throws ArithmeticException  wird geworfen, wenn versucht wird,
    *        eine Division durch Null durchzufhren.
   */
    public int gibWert(Variablenbelegung belegung) throws
                       UnbekannteVariableException, ArithmeticException {
        int ergebnis;
        
        int linkerWert = this.linkerTeilausdruck.gibWert(belegung);
        int rechterWert = this.rechterTeilausdruck.gibWert(belegung);
        
        if (this.operatorsymbol == PLUS) {
            ergebnis = linkerWert + rechterWert;
        } else if (this.operatorsymbol == MINUS) {
            ergebnis = linkerWert - rechterWert;
        } else if (this.operatorsymbol == MAL) {
            ergebnis = linkerWert * rechterWert;
        } else if (this.operatorsymbol == GETEILT) {
            if (rechterWert != 0) {
                ergebnis = linkerWert / rechterWert;
            } else {
                throw new ArithmeticException(DIVISIONDURCHNULL);
            }
        } else {
            /* kein gltiges operatorsymbol!! */
            ergebnis = 0;
        }
     
        return ergebnis;
    }
    
    /**
     * Stellt diesen Ausdruck in Postfix-Darstellung dar.
     *
     * @return  Postfix-Darstellung dieses Ausdrucks.
     */
    public String postfixDarstellung() {
        
        return this.linkerTeilausdruck.postfixDarstellung() + LEERZEICHEN
                + this.rechterTeilausdruck.postfixDarstellung() + LEERZEICHEN
                + this.operatorsymbol;
    }
    
    
}
