/*
 * Christoph Peters
 */

import java.util.StringTokenizer;
import java.text.ParseException;


/**
 * Parst die Textdarstellung eines Ausdrucks (bliche Infixdarstellung) und
 * liefert zugehrigen Ausdruck. (D.h. ein Objekt der Klasse Ausdruck)
 */
public class Parser {
    
    /** Trennzeichen, um einen Ausdruck in seine Basiselemente
     *  zerlegen zu knnen. (plus,minus,mal,geteilt,klammerauf,klammerzu,
     *  leerzeichen)*/
    private static final String TRENNZEICHEN = "+-*/() ";
    
    /** Trennzeichen, um die Basiselemente eines Ausdrucks unterscheiden
     *  zu knnen. */
    private static final String TRENNER = ",";
    
    /** Leerzeichen. */
    private static final String LEERZEICHEN = " ";
    
    /** Regulrer Ausdruck fr die Gltigkeit einer Konstante. */
    private static final String REGEXKONSTANTE = "[0-9]+";
    
    /** Regulrer Ausdruck fr die Gltigkeit einer Variablenbezeichnung. */
    private static final String REGEXVARIABLE = "[a-zA-Z][a-zA-Z0-9]*";
    
    /** Regulrer Ausdruck fr die Gltigkeit eines einzelnen
     * Summenzeichens. (plus oder minus)*/
    private static final String REGEXSUMMENOPERATOR = "[+|-]";
    
    /** Zeichen fr eine ffnende Klammer in einem Ausdruck. */
    private static final String KLAMMERAUF = "(";
    
    /** Zeichen fr eine schlieende Klammer in einem Ausdruck. */
    private static final String KLAMMERZU = ")";
    
    /** Multiplikationszeichen. */
    private static final String MULTI = "*";
    
    /** Divisionszeichen. */
    private static final String DIV = "/";
    
    /** Additionsszeichen. */
    private static final String PLUS = "+";
    
    /** Subtraktionszeichen. */
    private static final String MINUS = "-";
    
    /** Meldung bei ungltigem Token wenn ParseException geworfen wird. */
    private static final String UNGUELTIGESTOKEN = "ungueltiges Token";
    
    /** Meldung bei unerwartetem Ende wenn ParseException geworfen wird. */
    private static final String UNERWARTETESENDE = "unerwartetes Ende";

    /** String Array, in dem sie die einzelnen Tokens des Ausdrucks, whrend
     * des Parse-Vorgangs befinden. */
    private String[] tokens;
    
    /** Index-Zeiger auf das aktuell zu verarbeitene Token des Ausdrucks,
     * whrend des Parse-Vorgangs. */
    private int aktuellesToken;
    
    /** Erzeug Objekte dieser Klasse. */
    public Parser() {
        
    }
    

    /**
     * Unterteilt die Textdarstellung eines Ausdrucks
     * (bliche Infixdarstellung) in seine Basiselemente und gibt die
     * Kommagetrennt zurck. Leerzeichen werden dabei ignoriert.
     *
     * @param ausdruck  Textdarstellung den zu unterteilenden Ausdrucks.
     * @return  Basiselemente des bergebenen Ausdruck durch Kommata
     *          getrennt.
     */
    private String unterteile(String ausdruck) {
 
        /* StringTokenizer Objekt erzeugen, welches auch die
         * Delimiter als Tokens zurck gibt. */
        StringTokenizer tokenizer = new StringTokenizer(ausdruck,
                                                        TRENNZEICHEN, true);
        String unterteilterString = "";
        String nextToken;
        
        /* Solange iterieren, bis kein weiteres Token mehr vorhanden ist. */
        while (tokenizer.hasMoreTokens()) {
            //nchstes Token nehmen
            nextToken = tokenizer.nextToken();
            
            /* Nur wenn das Aktuelle Token kein Leerzeichen ist. */
            if  (!nextToken.equals(LEERZEICHEN)) {
                
                /* Token anfgen. */
                unterteilterString += nextToken;
                
                /* nur wenn noch ein weiteres Token existiert, darf ein 
                 * Trennzeichen (Komma) angehangen werden, nur nach dem
                 * letzen Token nicht. */
                if (tokenizer.hasMoreTokens()) {
                    unterteilterString += TRENNER;
                }
            }
        }
        
        return unterteilterString;
    }
    
    /**
     * Parst die Testdarstellung (gewhnliche Infixdarstellung) eines
     * Ausdrucks und gibt ein entsprechendes Ausdruck-Objekt zurck.
     * 
     * @param ausdruck        Textuelle Darstellung eines Ausdrucks (Infix).
     * @return                Entsprechendes Ausdruck-Objekt des
     *                        geparsten Ausdrucks.
     * @throws ParseException  Enthlt die Zeichenkette ausdruck ungltige
     *                        Konstanten, Variablenbezeichner oder endet
     *                        diese vorzeitig, wird diese Ausnahme geworfen.
     */
    public Ausdruck parse(String ausdruck) throws ParseException {
        
        Ausdruck gesamtAusdruck; //geparster gesamtausdruck
        
        /* basiselemente des Ausdrucks durch TRENNER getrennt in einem String
         * auflisten. Leerzeichen dabei ignorieren. */
        ausdruck = this.unterteile(ausdruck);
        
        int i = 0; //Zhlvariable zum initialisieren des Token Array.
        
        //Tokenizer der die durch TRENNER getrennten Tokens wieder trennt.
        StringTokenizer tokenizer = new StringTokenizer(ausdruck, TRENNER);
        
        //Fr jedes Token ein Arrayfeld reservieren.
        this.tokens = new String[tokenizer.countTokens()];
        
        /* Solange Tokens vorhanden sind, diese in die entsprechenden 
         * Arraypositionen kopieren */
        while (tokenizer.hasMoreTokens()) {
            this.tokens[i] = tokenizer.nextToken();
            i++;
        }
        
        //Parsevorgang beim nullten Token beginnen
        this.aktuellesToken = 0;
        
        //ausdruck parsen
        gesamtAusdruck = parseAusdruck();
        
        /* nur wenn alle Tokens verarbeitet wurden das Ergebnis-Objekt
         * zurck geben, sonst eine Exception werfen. bsp "(a+b)c" */
        if (this.aktuellesToken >= this.tokens.length) {
            return gesamtAusdruck;
        } else {
            /* ErrorOffset ist das aktuelle Token. */
            throw new ParseException(UNGUELTIGESTOKEN + LEERZEICHEN
                                            + this.tokens[aktuellesToken],
                                          aktuellesToken);
        }
    }
    
    /**
     * Parst einen Ausdruck. Dazu wird ber die einzelnen Tokens des
     * Objektinternen String Arrays tokens iteriert.
     *
     * @return                Ausdruck-Objekt des erkannten Ausdrucks.
     * @throws ParseException  Enthlt der Ausdruck ungltige Konstanten,
     *                        Variablenbezeichner oder endet dieser vorzeitig,
     *                        wird diese Ausnahme geworfen.
     */
    private Ausdruck parseAusdruck() throws ParseException {
        
        //ersten Summanden erkennen und Zwischenspeichern
        Ausdruck ausdruck1 = parseSummand(); 
        Ausdruck ausdruck2;
        char operator;
        
        /* Solange weitere Additionen/Subtraktionen in diesem Ausdruck
         * statt finden, diese erkennen und aufnehmen. Es werden immer zwei
         * Summanden zu einem Teilausdruck zusammengefasst. */
        while (istTokenSummenOperator()) {
            //Operatorsymbol zwischenspeichern
            operator = this.tokens[aktuellesToken].charAt(0);
            
            //operatorsymbol wurde verarbeitet, Indexzeiger weitersetzen
            aktuellesToken++;
            
            //Zweiten Summanden erkennen und Zwischenspeichern
            ausdruck2 = parseSummand();
            
            //Operatorausdruck aus den zwei Summanden bilden.
            ausdruck1 = new OperatorAusdruck(ausdruck1, operator,
                                                ausdruck2);
        }
        return ausdruck1;
    }
    
    /**
     * Prft, ob das Token an aktuell zu verarbeidender Position im 
     * Objektinternen Array ein Summenzeichen (Plus oder Minus) ist.
     *
     * @return  wenn ein Zeichen vorhanden ist und dieses einem Plus oder
     *         minus entspreicht true, sonst false.
     */
    private boolean istTokenSummenOperator() {
        
        /* Wenn ein weiteres Token existiert... , sonst false zurck geben. */
        if (aktuellesToken < this.tokens.length) {
            /*...Prfen ob das existierende Token ein plus oder minus zeichen
             * ist */
            return (this.tokens[aktuellesToken].matches(REGEXSUMMENOPERATOR));
        } else {
            return false;
        }
    }
   
     /**
     * Parst einen Summand. Dazu wird ber die einzelnen Tokens des
     * Objektinternen String Arrays tokens iteriert.
     *
     * @return                Ausdruck-Objekt des erkannten Summanden.
     * @throws ParseException  Enthlt der Summand ungltige Konstanten,
     *                        Variablenbezeichner oder endet dieser vorzeitig,
     *                        wird diese Ausnahme geworfen.
     */
    private Ausdruck parseSummand() throws ParseException {
        //ersten Faktor erkennen und Zwischenspeichern
        Ausdruck summand1 = parseFaktor();
        Ausdruck summand2;
        char operator;

        /*wenn ein noch ein Token existiert, weitere/n Faktor/en erkennen
         * und aufnehmen */
        if (aktuellesToken < this.tokens.length) {
            /* Solange weitere Multiplikationen/Divisionen in diesem Summand
             * statt finden, diese erkennen und aufnehmen. Es werden immer zwei
             * Faktoren zu einem Teilausdruck zusammengefasst. */
            while (istTokenProduktOperator()) {
                //Operatorsymbol zwischenspeichern
                operator = this.tokens[aktuellesToken].charAt(0);
                
                //Operatorsymbol wurde verarbeitet, Indexzeiger weitersetzen
                aktuellesToken++;
                
                //Zweiten Faktor erkennen und Zwischenspeichern
                summand2 = parseFaktor();
                
                //Operatorausdruck aus den zwei Faktoren bilden.
                summand1 = new OperatorAusdruck(summand1, operator, summand2);
            }
        }
        
        return summand1;
    }
    
    /**
     * Prft, ob das Token an aktuell zu verarbeidender Position im 
     * Objektinternen Array ein Produktzeichen (Mal oder geteilt) ist.
     *
     * @return  wenn ein Zeichen vorhanden ist und dieses einem mal oder
     *          geteil entspreicht true, sonst false.
     */
    private boolean istTokenProduktOperator() {
        /* Wenn ein weiteres Token existiert... , sonst false zurck geben. */
        if (aktuellesToken < this.tokens.length) {
            /*...Prfen ob das existierende Token ein mal oder geteilt zeichen
             * ist */
            return this.tokens[aktuellesToken].matches("[" + MULTI + "|" 
                                                  + DIV + "]");
        } else {
            return false;
        }
    }
    
    /**
     * Parst einen Faktor. Dazu wird ber die einzelnen Tokens des
     * Objektinternen String Arrays tokens iteriert.
     *
     * @return                Ausdruck-Objekt des erkannten Faktors.
     * @throws ParseException  Enthlt der Faktor ungltige Konstanten,
     *                        Variablenbezeichner oder endet dieser vorzeitig,
     *                        wird diese Ausnahme geworfen.
     */
    private Ausdruck parseFaktor() throws ParseException {
        Ausdruck faktor;
        int wertKonstante;
        
        //Wenn aktuelles Token existiert
        if (this.aktuellesToken < this.tokens.length) {

            /* wenn aktuelles Token eine Konstante ist, dann Wert
             * ermitteln und Konstantenobjekt erzeugen. */
            if (this.tokens[aktuellesToken].matches(REGEXKONSTANTE)) {
                wertKonstante = Integer.parseInt(this.tokens[aktuellesToken]);
                faktor = new Konstante(wertKonstante);   
                
            } else if (this.tokens[aktuellesToken].matches(REGEXVARIABLE)) {
                /* wenn aktuelles Token eine Variable ist, dann
                 * Variablenobjekt erzeugen */
                faktor = new Variable(this.tokens[aktuellesToken]);
                
            } else if (this.tokens[aktuellesToken].equals(KLAMMERAUF)) {
                /* klammerAUFtoken wurde erkannt und verarbeitet,
                 * also TokenZeiger weiter */
                aktuellesToken++;
                
                /* Ausdruck zwischen den Klammern erkennen und
                 * zwischenspeichern */
                faktor = parseAusdruck();
                
                /* wenn ein weiteres Token existiert */
                if (this.aktuellesToken < this.tokens.length) {
                    
                    if (!(this.tokens[aktuellesToken].equals(KLAMMERZU))) {
                        /* wenn das aktuelle Token keine schlieende runde
                         * Klammer ist, wird Exception geworfen.
                         * ErrorOffset ist das aktuelle Token. */
                        throw new ParseException(UNGUELTIGESTOKEN 
                                                 + LEERZEICHEN 
                                                 + this.tokens[aktuellesToken],
                                           aktuellesToken);
                    }
                } else {
                    /* wenn kein zeichen mehr kommt.
                     * ErrorOffset ist hier irrelevant. */
                    throw new ParseException(UNERWARTETESENDE, 0);
                }
            } else {
                /* Aktuelles Token ist keine Variable, keine Konstante und
                 * auch keine ffnende runde Klammer, also wird Exception
                 * geworfen.
                 * ErrorOffset ist das aktuelle Token. */
                throw new ParseException(UNGUELTIGESTOKEN + LEERZEICHEN
                                         + this.tokens[aktuellesToken],
                                    aktuellesToken);
            }
        } else {
            /* Es sollte in dieser Methode ein Faktor geparst werden, jedoch
             * existiert kein weiteres Token mehr.
             * ErrorOffset ist hier irrelevant. */
            throw new ParseException(UNERWARTETESENDE, 0);
        }
        
        //faktor wurde erkannt und verarbeitet, Indexzeiger weitersetzen
        this.aktuellesToken++;
        
        return faktor;
    }
    

}
